@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT:
  * Copyright (c) 1997-2016, International Business Machines Corporation and
@@ -26,6 +28,7 @@
 #include "unicode/uloc.h"
 #include "unicode/uchar.h"
 #include "unicode/ustring.h"
+#include "unicode/utf16.h"
 #include "unicode/putil.h"
 #include "callcoll.h"
 #include "cmemory.h"
@@ -1073,7 +1076,7 @@ static void TestDiscontiguos() {
         log_verbose("count %d\n", count);
 
         for (;;) {
-            uint32_t  ce;
+            int32_t  ce;
             UChar    *e = u_strchr(s, 0x20);
             if (e == 0) {
                 e = u_strchr(s, 0);
@@ -1085,7 +1088,7 @@ static void TestDiscontiguos() {
                 return;
             }
             while (ce != UCOL_NULLORDER) {
-                if (ce != (uint32_t)ucol_next(iter, &status) ||
+                if (ce != ucol_next(iter, &status) ||
                     U_FAILURE(status)) {
                     log_err("Discontiguos contraction test mismatch\n");
                     return;
