@@ -454,6 +454,7 @@ static void
 testLCID(UResourceBundle *currentBundle,
          const char *localeName)
 {
+    (void)currentBundle; // suppress compiler warnings about unused variables
     UErrorCode status = U_ZERO_ERROR;
     uint32_t expectedLCID;
     char lcidStringC[64] = {0};
@@ -1028,11 +1029,6 @@ static void VerifyTranslation(void) {
                     end = ures_getSize(resArray);
                 }
 
-                if ((uprv_strcmp(currLoc,"sd") == 0 || uprv_strncmp(currLoc,"sd_",3) == 0) && 
-                        log_knownIssue("cldrbug:11491", "sd month 6 has char not in exemplars")) {
-                    end = 0;
-                }
-
                 for (idx = 0; idx < end; idx++) {
                     const UChar *fromBundleStr = ures_getStringByIndex(resArray, idx, &langSize, &errorCode);
                     if (U_FAILURE(errorCode)) {
