@@ -33,6 +33,8 @@ private:
     void testSkeletonsWithDayPeriods();
     void testGetFieldDisplayNames();
     void testJjMapping();
+    void test20640_HourCyclArsEnNH();
+    void testFallbackWithDefaultRootLocale();
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
